import unittest
import math
import numpy

import irwin


class CanvasTest(unittest.TestCase):

    def assertCanvas(self, actual, expected):
        if actual != expected:
            print('Actual:')
            print(actual)
            print('Expected:')
            print(expected)
            self.fail()

    def test_render_empty_canvas(self):
        canvas = irwin.Canvas(15, 10, 0, 1, 0, 1)

        self.assertCanvas(canvas.render(),
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀')

    def test_draw_point(self):
        canvas = irwin.Canvas(2, 2, 0, 1, 0, 1)

        canvas.draw_point(0, 0)
        canvas.draw_point(0, 1)
        canvas.draw_point(1, 0)
        canvas.draw_point(1, 1)

        self.assertCanvas(canvas.render(),
                          '⠁⠈\n'
                          '⡀⢀')

    def test_draw_line(self):
        canvas = irwin.Canvas(10, 10, 0, 2, 0, 2)

        canvas.draw_line(0, 0, 1, 1)
        canvas.draw_line(2, 0, 1, 1)

        canvas.draw_line(0.5, 0, 1, 1)
        canvas.draw_line(1.5, 0, 1, 1)

        canvas.draw_line(0, 0, 2, 2)

        canvas.draw_line(0, 0, 0, 2)
        canvas.draw_line(0, 2, 2, 2)
        canvas.draw_line(2, 2, 2, 0)
        canvas.draw_line(2, 0, 0, 0)

        self.assertCanvas(canvas.render(),
                          '⡏⠉⠉⠉⠉⠉⠉⠉⠉⣽\n'
                          '⡇⠀⠀⠀⠀⠀⠀⠀⡜⢸\n'
                          '⡇⠀⠀⠀⠀⠀⠀⡜⠀⢸\n'
                          '⡇⠀⠀⠀⠀⠀⡜⠀⠀⢸\n'
                          '⡇⠀⠀⠀⠀⡜⠀⠀⠀⢸\n'
                          '⡇⠀⠀⠀⣼⡄⠀⠀⠀⢸\n'
                          '⡇⠀⠀⣼⠃⢟⢄⠀⠀⢸\n'
                          '⡇⠀⡜⡜⠀⠘⡌⢆⠀⢸\n'
                          '⡇⡜⢰⠁⠀⠀⢱⠈⢢⢸\n'
                          '⣟⣀⣇⣀⣀⣀⣀⣇⣀⣻')

    def test_draw_line_odd_height(self):
        canvas = irwin.Canvas(10, 9, 0, 2, 0, 2)

        canvas.draw_line(0, 0, 1, 1)
        canvas.draw_line(2, 0, 1, 1)

        canvas.draw_line(0.5, 0, 1, 1)
        canvas.draw_line(1.5, 0, 1, 1)

        canvas.draw_line(0, 0, 2, 2)

        canvas.draw_line(0, 0, 0, 2)
        canvas.draw_line(0, 2, 2, 2)
        canvas.draw_line(2, 2, 2, 0)
        canvas.draw_line(2, 0, 0, 0)

        self.assertCanvas(canvas.render(),
                          '⡏⠉⠉⠉⠉⠉⠉⠉⢉⢿\n'
                          '⡇⠀⠀⠀⠀⠀⠀⢀⠎⢸\n'
                          '⡇⠀⠀⠀⠀⠀⢠⠊⠀⢸\n'
                          '⡇⠀⠀⠀⠀⢠⠃⠀⠀⢸\n'
                          '⡇⠀⠀⠀⣠⠃⠀⠀⠀⢸\n'
                          '⡇⠀⠀⡰⡏⡧⡀⠀⠀⢸\n'
                          '⡇⠀⡰⡹⠀⠸⡑⡄⠀⢸\n'
                          '⡇⡜⢠⠃⠀⠀⢣⠈⢆⢸\n'
                          '⣟⣀⣎⣀⣀⣀⣈⣆⣀⣻')

    def test_draw_line_partly_outside_canvas(self):
        canvas = irwin.Canvas(10, 10, 0, 2, 0, 2)

        canvas.draw_line(-1000000000, -1000000000, 3000000000, 3000000000)

        self.assertCanvas(canvas.render(),
                          '⠀⠀⠀⠀⠀⠀⠀⠀⢀⠎\n'
                          '⠀⠀⠀⠀⠀⠀⠀⢀⠎⠀\n'
                          '⠀⠀⠀⠀⠀⠀⢀⠎⠀⠀\n'
                          '⠀⠀⠀⠀⠀⢀⠎⠀⠀⠀\n'
                          '⠀⠀⠀⠀⢀⠎⠀⠀⠀⠀\n'
                          '⠀⠀⠀⢀⠎⠀⠀⠀⠀⠀\n'
                          '⠀⠀⢀⠎⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⡜⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⡜⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⡜⠀⠀⠀⠀⠀⠀⠀⠀⠀')

    def test_draw_line_partly_outside_canvas_2(self):
        canvas = irwin.Canvas(10, 10, 0, 2, 0, 2)

        canvas.draw_line(-1, -3, 3, 5)

        self.assertCanvas(canvas.render(),
                          '⠀⠀⠀⠀⠀⠀⢀⠇⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⡸⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⢀⠇⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⡸⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⡇⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⢸⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⡇⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⢸⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⡇⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⢸⠀⠀⠀⠀⠀⠀⠀')

    def test_draw_line_partly_outside_canvas_3(self):
        canvas = irwin.Canvas(10, 10, 0, 2, 0, 2)

        canvas.draw_line(-1, 5, 3, -3)

        self.assertCanvas(canvas.render(),
                          '⠀⠀⢸⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⡇⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⢸⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⡇⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⢸⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⡇⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⢸⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⡇⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⢸⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⡇⠀⠀')

    def test_draw_line_partly_outside_canvas_4(self):
        canvas = irwin.Canvas(10, 10, 0, 2, 0, 2)

        canvas.draw_line(-2, 0, -1, 0)

        self.assertCanvas(canvas.render(),
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀')

    def test_draw_line_partly_outside_canvas_5(self):
        canvas = irwin.Canvas(10, 10, 0, 2, 0, 2)

        canvas.draw_line(1, 1, 3, -1)

        self.assertCanvas(canvas.render(),
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠘⡄⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠘⡄⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠘⡄⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠘⡄⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠘⢄')

    def test_draw_line_length_zero(self):
        canvas = irwin.Canvas(10, 10, 0, 2, 0, 2)

        canvas.draw_line(1, 1, 1, 1)

        self.assertCanvas(canvas.render(),
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠈⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀')

    def test_draw(self):
        canvas = irwin.Canvas(40, 20, 0, 10, 0, 10)

        # Draw the canvas edges.
        canvas.draw_line(0, 0, 0, 10)
        canvas.draw_line(0, 10, 10, 10)
        canvas.draw_line(10, 10, 10, 0)
        canvas.draw_line(10, 0, 0, 0)

        # Draw a cross.
        canvas.draw_line(0, 0, 10, 10)
        canvas.draw_line(0, 10, 10, 0)

        # Draw four points.
        canvas.draw_point(5, 2.5)
        canvas.draw_point(5, 7.5)
        canvas.draw_point(2.5, 5)
        canvas.draw_point(7.5, 5)

        self.assertCanvas(canvas.render(),
                          '⡟⢍⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⠉⡩⢻\n'
                          '⡇⠀⠑⢄⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡠⠊⠀⢸\n'
                          '⡇⠀⠀⠀⠑⢄⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡠⠊⠀⠀⠀⢸\n'
                          '⡇⠀⠀⠀⠀⠀⠑⢄⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡠⠊⠀⠀⠀⠀⠀⢸\n'
                          '⡇⠀⠀⠀⠀⠀⠀⠀⠑⢄⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡠⠊⠀⠀⠀⠀⠀⠀⠀⢸\n'
                          '⡇⠀⠀⠀⠀⠀⠀⠀⠀⠀⠑⢄⠀⠀⠀⠀⠀⠀⠀⠈⠀⠀⠀⠀⠀⠀⠀⠀⡠⠊⠀⠀⠀⠀⠀⠀⠀⠀⠀⢸\n'
                          '⡇⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠑⢄⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡠⠊⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢸\n'
                          '⡇⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠑⢄⠀⠀⠀⠀⠀⠀⠀⠀⡠⠊⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢸\n'
                          '⡇⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠑⢄⠀⠀⠀⠀⡠⠊⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢸\n'
                          '⡇⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠑⢄⡠⠊⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢸\n'
                          '⡇⠀⠀⠀⠀⠀⠀⠀⠀⠈⠀⠀⠀⠀⠀⠀⠀⠀⡠⠊⠑⢄⠀⠀⠀⠀⠀⠀⠀⠈⠀⠀⠀⠀⠀⠀⠀⠀⠀⢸\n'
                          '⡇⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡠⠊⠀⠀⠀⠀⠑⢄⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢸\n'
                          '⡇⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡠⠊⠀⠀⠀⠀⠀⠀⠀⠀⠑⢄⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢸\n'
                          '⡇⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡠⠊⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠑⢄⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢸\n'
                          '⡇⠀⠀⠀⠀⠀⠀⠀⠀⠀⡠⠊⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠑⢄⠀⠀⠀⠀⠀⠀⠀⠀⠀⢸\n'
                          '⡇⠀⠀⠀⠀⠀⠀⠀⡠⠊⠀⠀⠀⠀⠀⠀⠀⠀⠀⠈⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠑⢄⠀⠀⠀⠀⠀⠀⠀⢸\n'
                          '⡇⠀⠀⠀⠀⠀⡠⠊⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠑⢄⠀⠀⠀⠀⠀⢸\n'
                          '⡇⠀⠀⠀⡠⠊⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠑⢄⠀⠀⠀⢸\n'
                          '⡇⠀⡠⠊⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠑⢄⠀⢸\n'
                          '⣧⣊⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣀⣑⣼')

    def test_render_methods(self):
        canvas = irwin.Canvas(40, 20, 0, 2 * math.pi, -1, 1)
        x = numpy.linspace(0, 2 * math.pi)
        y = numpy.sin(x)
        canvas.draw_lines(x, y)

        self.assertCanvas(canvas.render(),
                          '⠀⠀⠀⠀⠀⠀⠀⢀⠔⠒⠒⠢⡀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⡔⠁⠀⠀⠀⠀⠘⢄⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⡜⠀⠀⠀⠀⠀⠀⠀⠈⢢⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⡸⠀⠀⠀⠀⠀⠀⠀⠀⠀⠈⡆⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⢠⠃⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠈⡆⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⢠⠃⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢱⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⡇⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠈⢆⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠀⢸⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠘⢄⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⢠⠃⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠸⡀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⡜⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢇⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n'
                          '⠁⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠈⡆⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡜\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢣⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡸⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠘⢄⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢰⠁⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠘⡄⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡎⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢸⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡜⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡇⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡰⠁⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠈⡆⠀⠀⠀⠀⠀⠀⠀⠀⠀⢠⠃⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠘⢄⠀⠀⠀⠀⠀⠀⠀⢠⠃⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠈⢢⠀⠀⠀⠀⠀⢀⠎⠀⠀⠀⠀⠀⠀\n'
                          '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠣⣀⣀⣀⠔⠁⠀⠀⠀⠀⠀⠀⠀')

        self.assertEqual(
            list(canvas.render_lines()),
            [
                '⠀⠀⠀⠀⠀⠀⠀⢀⠔⠒⠒⠢⡀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀',
                '⠀⠀⠀⠀⠀⠀⡔⠁⠀⠀⠀⠀⠘⢄⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀',
                '⠀⠀⠀⠀⠀⡜⠀⠀⠀⠀⠀⠀⠀⠈⢢⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀',
                '⠀⠀⠀⠀⡸⠀⠀⠀⠀⠀⠀⠀⠀⠀⠈⡆⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀',
                '⠀⠀⠀⢠⠃⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠈⡆⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀',
                '⠀⠀⢠⠃⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢱⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀',
                '⠀⠀⡇⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠈⢆⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀',
                '⠀⢸⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠘⢄⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀',
                '⢠⠃⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠸⡀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀',
                '⡜⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢇⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀',
                '⠁⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠈⡆⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡜',
                '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢣⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡸⠀',
                '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠘⢄⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢰⠁⠀',
                '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠘⡄⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡎⠀⠀',
                '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⢸⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡜⠀⠀⠀',
                '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡇⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⡰⠁⠀⠀⠀',
                '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠈⡆⠀⠀⠀⠀⠀⠀⠀⠀⠀⢠⠃⠀⠀⠀⠀',
                '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠘⢄⠀⠀⠀⠀⠀⠀⠀⢠⠃⠀⠀⠀⠀⠀',
                '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠈⢢⠀⠀⠀⠀⠀⢀⠎⠀⠀⠀⠀⠀⠀',
                '⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀⠣⣀⣀⣀⠔⠁⠀⠀⠀⠀⠀⠀⠀'

            ])

        self.assertEqual(list(canvas.render_segments()),
                         [
                             (0, 7, '⢀⠔⠒⠒⠢⡀'),
                             (1, 6, '⡔⠁'), (1, 12, '⠘⢄'),
                             (2, 5, '⡜'), (2, 13, '⠈⢢'),
                             (3, 4, '⡸'), (3, 14, '⠈⡆'),
                             (4, 3, '⢠⠃'), (4, 15, '⠈⡆'),
                             (5, 2, '⢠⠃'), (5, 16, '⢱'),
                             (6, 2, '⡇'), (6, 16, '⠈⢆'),
                             (7, 1, '⢸'), (7, 17, '⠘⢄'),
                             (8, 0, '⢠⠃'), (8, 18, '⠸⡀'),
                             (9, 0, '⡜'), (9, 19, '⢇'),
                             (10, 0, '⠁'), (10, 19, '⠈⡆'), (10, 39, '⡜'),
                             (11, 20, '⢣'), (11, 38, '⡸'),
                             (12, 20, '⠘⢄'), (12, 37, '⢰⠁'),
                             (13, 21, '⠘⡄'), (13, 37, '⡎'),
                             (14, 22, '⢸'), (14, 36, '⡜'),
                             (15, 23, '⡇'), (15, 35, '⡰⠁'),
                             (16, 23, '⠈⡆'), (16, 34, '⢠⠃'),
                             (17, 24, '⠘⢄'), (17, 33, '⢠⠃'),
                             (18, 25, '⠈⢢'), (18, 32, '⢀⠎'),
                             (19, 27, '⠣⣀⣀⣀⠔⠁')
                         ])

    def test_draw_rectangle(self):
        canvas = irwin.Canvas(10, 10, 0, 2, 0, 2)

        canvas.draw_rectangle(0, 0, 2, 2)

        self.assertCanvas(canvas.render(),
                          "⡏⠉⠉⠉⠉⠉⠉⠉⠉⢹\n"
                          "⡇⠀⠀⠀⠀⠀⠀⠀⠀⢸\n"
                          "⡇⠀⠀⠀⠀⠀⠀⠀⠀⢸\n"
                          "⡇⠀⠀⠀⠀⠀⠀⠀⠀⢸\n"
                          "⡇⠀⠀⠀⠀⠀⠀⠀⠀⢸\n"
                          "⡇⠀⠀⠀⠀⠀⠀⠀⠀⢸\n"
                          "⡇⠀⠀⠀⠀⠀⠀⠀⠀⢸\n"
                          "⡇⠀⠀⠀⠀⠀⠀⠀⠀⢸\n"
                          "⡇⠀⠀⠀⠀⠀⠀⠀⠀⢸\n"
                          "⣇⣀⣀⣀⣀⣀⣀⣀⣀⣸")

    def test_calc_point_row_and_col(self):
        canvas = irwin.Canvas(10, 10, 0, 2, 0, 2)

        canvas.draw_point(0, 0)
        canvas.draw_point(0, 2)
        canvas.draw_point(2, 0)
        canvas.draw_point(2, 2)
        canvas.draw_point(0.5, 0.5)

        self.assertCanvas(canvas.render(),
                          "⠁⠀⠀⠀⠀⠀⠀⠀⠀⠈\n"
                          "⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n"
                          "⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n"
                          "⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n"
                          "⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n"
                          "⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n"
                          "⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n"
                          "⠀⠀⠄⠀⠀⠀⠀⠀⠀⠀\n"
                          "⠀⠀⠀⠀⠀⠀⠀⠀⠀⠀\n"
                          "⡀⠀⠀⠀⠀⠀⠀⠀⠀⢀")

        self.assertEqual(canvas.calc_point_row_and_col(0, 0), (9, 0))
        self.assertEqual(canvas.calc_point_row_and_col(0, 2), (0, 0))
        self.assertEqual(canvas.calc_point_row_and_col(2, 0), (9, 9))
        self.assertEqual(canvas.calc_point_row_and_col(2, 2), (0, 9))
        self.assertEqual(canvas.calc_point_row_and_col(0.5, 0.5), (7, 2))
